// Rock, paper, scissors friend
// Designed by Raquel Sarabia
// Date: 07 October 2021
// ---------------------------------------------------------------- //
// Description:
// This program plays the game Rock, Paper, Scissors
// If the ultrasonic sensor detects a hand at a close distance it generates
// a random movement (rock, paper or scissors) and asks the user what has
// been the result. The result is introduced by pushing the respective button:
// P1 if player one won, P2 if player two won or tie if both were equal.
// After this when the ultrasonic sensor detects the player again round 2 starts.
// The game finish when one of the player reaches 3 points and it starts again.
// ---------------------------------------------------------------- //

#define PIN_A0   (14)//Use analog pin as digital
#define PIN_A1   (15)//Use analog pin as digital
#define PIN_A2   (16)//Use analog pin as digital
#define echoPin 4 // attach pin D2 Arduino to pin Echo of HC-SR04
#define trigPin 5 //attach pin D3 Arduino to pin Trig of HC-SR04
#include <Servo.h> // load servo library
Servo myservo1; // create servo object
Servo myservo2; // create servo object

// defines variables
long duration; // variable for the duration of sound wave travel
int distance;// variable for the distance measurement
long aleatorio;//random varible that will generate a new move
long paleat;// variable to save previous rand number
int angle1; // declare variable for myservo1 angle1
int angle2; // declare variable for myservo2 angle2
int k = 1; // variable to initialice
int p1 = 0; // variable to define player's 1 result
int p2 = 0; // variable to define player's 2 result

void setup() {
  pinMode(trigPin, OUTPUT); // Sets the trigPin as an OUTPUT
  pinMode(echoPin, INPUT); // Sets the echoPin as an INPUT
  Serial.begin(9600); // // Serial Communication is starting with 9600 of baudrate speed
  myservo1.attach(11); // tell the Arduino which pin will drive the servo
  myservo2.attach(10); // tell the Arduino which pin will drive the servo
  pinMode(7, OUTPUT); // Green LED P1 (1st point)
  pinMode(6, OUTPUT); // Green LED P1 (2nd point)
  pinMode(3, OUTPUT); // Green LED P1 (3rd point)
  pinMode(14, OUTPUT); // Green LED P2 (1st point)
  pinMode(15, OUTPUT); // Green LED P1 (2nd point)
  pinMode(2, OUTPUT); // Green LED P2 (3rd point)
  pinMode(9, INPUT); // Button P1
  pinMode(8, INPUT); // Button P2
  pinMode(16, INPUT); // Button tie
  pinMode(12, OUTPUT); // Green LED start signal
  pinMode(13, OUTPUT); // Red LED ready to start signal
  randomSeed(analogRead(A3));//Instruction I've read that avoids random command from being pseudo-random
}

void loop() {
  if (k == 1){ // set the program to a initial point (servos on a specific position and LEDs off)
    myservo1.write(10); // Initial position for myservo1
    myservo2.write(10); // Initial position for myservo1
    //Turn off every LED that shows the results
    digitalWrite(14,LOW); 
    digitalWrite(15,LOW);
    digitalWrite(2,LOW);
    digitalWrite(7,LOW);
    digitalWrite(6,LOW);
    digitalWrite(3,LOW);
    paleat = 2; // The initial position will be 'paper'
    k = k+1; // We avoid reading this in the following rounds
  }
  
  // Clears the trigPin condition
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  // Sets the trigPin HIGH (ACTIVE) for 10 microseconds
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  // Reads the echoPin, returns the sound wave travel time in microseconds
  duration = pulseIn(echoPin, HIGH);
  // Calculating the distance
  distance = duration * 0.034 / 2; // Speed of sound wave divided by 2 (go and back)
  // Displays the distance on the Serial Monitor
  Serial.print("Distance: ");
  Serial.print(distance);
  Serial.println(" cm");
  
  //Ready, set, go signal (green and red LEDs)
  digitalWrite(13, HIGH); //Turns on red
  delay(300);
  digitalWrite(13,LOW); //Turns off red
  delay(300);
  digitalWrite(13, HIGH); //Turns on red
  delay(300);
  digitalWrite(13,LOW);  //Turns off red
  delay(300);
  digitalWrite(13, HIGH); //Turns on red
  delay(300);
  digitalWrite(13,LOW); //Turns off red
  delay(300);
  digitalWrite(12, HIGH); //Turns on green
  delay(700);
  digitalWrite(12,LOW); //Turns off green
  
  //Here is where the game really starts
  if (4 <= distance && distance <=25){ //If the sensor detects the hand...
    aleatorio=random(1,4); // A random number is generated
    Serial.print(aleatorio);
    if (aleatorio == 1){
      Serial.println(" (ROCK)"); // Number 1 means ROCK
    }
    else if  (aleatorio == 2){
      Serial.println(" (PAPER)"); // Number 2 means paper
    }
    else{
      Serial.println(" (SCISSORS)"); // Number 3 means paper
    }
    
    //Code to do ROCK
    if (aleatorio == 1 && paleat == 1){ // If previous position was ROCK it stays the same
    }
    else if (aleatorio == 1 && paleat == 2){ // If previous was PAPER...
        for(int angle1 = 10; angle1 < 120; angle1++){ // Thumb, ring & little fingers close   CERRAR ++                           
        myservo1.write(angle1);  //servo spins         
        delay(2);
        }
        for(int angle2 = 10; angle2 < 120; angle2++){ // Index & middle fingers close  CERRAR ++                          
        myservo2.write(angle2); //servo spins           
        delay(2);       
      }
    }
    else if (aleatorio == 1 && paleat == 3){ // If previous was SCISSORS...
        for(int angle2 = 10; angle2 < 120; angle2++){ // Index & middle fingers close    CERRAR ++                          
        myservo2.write(angle2); //servo spins           
        delay(2);       
      }
    }
    
    //Code to do PAPER
    else if (aleatorio == 2 && paleat == 2){ // If previous position was PAPER it stays the same
    }
    else if (aleatorio == 2 && paleat == 1){ // If previous was ROCK...
        for(int angle1 = 180; angle1 > 10; angle1--){ // Thumb, ring & little fingers open      ABRIR --                          
        myservo1.write(angle1); //servo spins               
        delay(2);                   
      }
        for(int angle2 = 180; angle2 > 10; angle2--){ // Index & middle fingers open       ABRIR --                          
        myservo2.write(angle2); //servo spins               
        delay(2);
      }
    }
    else if (aleatorio == 2 && paleat == 3){ // If previous was SCISSORS...
        for(int angle1 = 180; angle1 > 10; angle1--){ // Thumb, ring & little fingers open       ABRIR --                          
        myservo1.write(angle1); //servo spins               
        delay(2);                   
      }
    }
    
    //Code to do SCISSORS
    else if (aleatorio == 3 && paleat == 3){// If previous position was SCISSORS it stays the same
    }
    else if (aleatorio == 3 && paleat == 1){// If previous was ROCK...
        for(int angle2 = 180; angle2 > 10; angle2--){ // Index & middle fingers open        ABRIR --                         
        myservo2.write(angle2); //servo spins               
        delay(2);
      }
    }
    else if (aleatorio == 3 && paleat == 2){// If previous was PAPER...
        for(int angle1 = 10; angle1 < 120; angle1++){ // Thumb, ring & little fingers close      CERRAR ++                         
        myservo1.write(angle1); //servo spins           
        delay(2);       
      }
    }
    
    Serial.println("Write the result");//Here you have to say what was the result to continue playing
    
    while (digitalRead(9) == LOW && digitalRead(8) == LOW && digitalRead(16) == LOW){//The codes blocks until you answer
    }
    if (digitalRead(9) == HIGH){//If P1 button is pulsed
      p1 = p1 + 1;//This adds a point to player one
      if (p1 == 1){
        digitalWrite(7, HIGH);// P1 1st point (LED)
      }
      else if (p1 == 2){
        digitalWrite(6, HIGH);// P1 2nd point (LED)
      }
      else if (p1 == 3){
        digitalWrite(3, HIGH);// P1 2nd point (LED)
        delay(1000);
        //Restart the scores of both players when one of them wins
        digitalWrite(7,LOW);
        digitalWrite(6,LOW);
        digitalWrite(3,LOW);
        digitalWrite(14,LOW);
        digitalWrite(15,LOW);
        digitalWrite(2,LOW);
        p1 = 0;
        p2 = 0;
      }
    }
    if (digitalRead(8) == HIGH){//If P2 button is pulsed
      p2 = p2 + 1;//This adds a point to player two
      if (p2 == 1){
        digitalWrite(14, HIGH);// P2 1st point (LED)
      }
      else if (p2 == 2){
        digitalWrite(15, HIGH);// P2 2nd point (LED)
      }
      else if (p2 == 3){
        digitalWrite(2, HIGH);// P2 3rd point (LED)
        delay(1000);
        //Restart the scores of both players when one of them wins
        digitalWrite(7,LOW);
        digitalWrite(6,LOW);
        digitalWrite(3,LOW);
        digitalWrite(14,LOW);
        digitalWrite(15,LOW);
        digitalWrite(2,LOW);
        p1 = 0;
        p2 = 0;
      }
    }
    if (digitalRead(16) == HIGH){//If tie button is pulsed
      Serial.println("tie");//Game continues
    }
    delay(2500);
    
    paleat=aleatorio;//paleat remembers the previous figure for the next round
    }    
}
